/*
 * Copyright (c) 2011 Mysema Ltd.
 * All rights reserved.
 *
 */
package com.mysema.commons.jetty;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.DefaultHandler;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.server.ssl.SslSelectChannelConnector;
import org.eclipse.jetty.webapp.WebAppContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * JettyHelper provides convenience methods for starting Jetty 7 instances with
 * compact configuration options
 *
 * @author Timo Westkamper
 * @author Marko Lavikainen
 * @version $Id$
 */
public final class JettyHelper {

    private JettyHelper() {
    }

    private static final Logger logger = LoggerFactory
            .getLogger(JettyHelper.class);

    private static final Map<Integer, Server> servers = new HashMap<Integer, Server>();

    private static final File keystoreLocation;

    static {
        try {
            keystoreLocation = File.createTempFile("keystore", null);
            InputStream in = JettyHelper.class.getResourceAsStream("/keystore");
            OutputStream out = new FileOutputStream(keystoreLocation);
            try {
                IOUtils.copy(in, out);
            } finally {
                out.close();
                in.close();
            }
        } catch (IOException e) {
            throw new JettyException(e.getMessage(), e);
        }

    }

    public static boolean runningAt(int port) {
        return servers.containsKey(port);
    }

    public static WebAppContext startJetty(String warPath, String contextPath,
            int port, int securePort) throws Exception { // NOPMD
        WebAppContext webappcontext = new WebAppContext();
        webappcontext.setContextPath(contextPath);
        webappcontext.setWar(warPath);
        if (new File(warPath).isFile()) {
            webappcontext.setExtractWAR(true);
        }
        startJetty(port, securePort, webappcontext);
        return webappcontext;
    }

    @SuppressWarnings("deprecation")
    public static void startJetty(int port, int securePort,
            WebAppContext... contexts) throws Exception { // NOPMD
        if (servers.containsKey(port)) {
            logger.error("Already one Jetty instance running on port " + port,
                    new JettyException());
            return;
        }

        if (contexts == null || contexts.length == 0) {
            throw new IllegalArgumentException();
        }

        final Server server = new Server();
        servers.put(port, server);

        List<Connector> connectors = new ArrayList<Connector>(2);

        // http
        if (port > 0) {
            SelectChannelConnector httpConnector = new SelectChannelConnector();
            httpConnector.setPort(port);
            httpConnector.setConfidentialPort(securePort);
            connectors.add(httpConnector);
        }

        // https
        if (securePort > 0) {
            SslSelectChannelConnector httpsConnector = new SslSelectChannelConnector();
            httpsConnector.setPort(securePort);
            httpsConnector.setMaxIdleTime(60000);

            /*
             * These methods are deprecated but could not find any alternatives
             * for them
             */
            httpsConnector.setKeystore(keystoreLocation.getAbsolutePath());
            httpsConnector.setPassword("OBF:1vny1zlo1x8e1vnw1vn61x8g1zlu1vn4");
            httpsConnector.setKeyPassword("OBF:1u2u1wml1z7s1z7a1wnl1u2g");
            connectors.add(httpsConnector);
        }

        server.setConnectors(connectors
                .toArray(new Connector[connectors.size()]));

        HandlerList handlers = new HandlerList();

        for (WebAppContext context : contexts) {
            handlers.addHandler(context);
        }

        handlers.addHandler(new DefaultHandler());

        server.setHandler(handlers);
        server.start();
        logger.info("Jetty running on port " + port);
    }

    public static void stopJettyAtPort(int port) throws Exception { // NOPMD
        Server server = servers.remove(port);
        if (server != null) {
            server.stop();
            logger.info("Stopped Jetty on port " + port);
        } else {
            logger.warn("No Jetty instance running on port " + port,
                    new JettyException());
        }
    }

}
